@@ -1,7 +1,7 @@
 /*
  * Create the trigger function for the parent table of a time-based partition set
  */
-CREATE FUNCTION create_function_time(p_parent_table text, p_job_id bigint DEFAULT NULL) RETURNS void
+CREATE OR REPLACE FUNCTION create_function_time(p_parent_table text, p_job_id bigint DEFAULT NULL) RETURNS void
     LANGUAGE plpgsql SECURITY DEFINER
     AS $$
 DECLARE
@@ -21,6 +21,7 @@ v_function_name                 text;
 v_job_id                        bigint;
 v_jobmon                        boolean;
 v_jobmon_schema                 text;
+v_new_search_path               text := '@extschema@,pg_temp';
 v_old_search_path               text;
 v_new_length                    int;
 v_next_partition_name           text;
@@ -33,7 +34,9 @@ v_prev_partition_timestamp      timestamptz;
 v_step_id                       bigint;
 v_trig_func                     text;
 v_optimize_trigger              int;
+v_trigger_exception_handling    boolean;
 v_type                          text;
+v_upsert                        text;
 
 BEGIN
 
@@ -44,6 +47,8 @@ SELECT partition_type
     , optimize_trigger
     , datetime_string
     , jobmon
+    , trigger_exception_handling
+    , upsert
 INTO v_type
     , v_partition_interval
     , v_epoch
@@ -51,6 +56,8 @@ INTO v_type
     , v_optimize_trigger
     , v_datetime_string
     , v_jobmon
+    , v_trigger_exception_handling
+    , v_upsert
 FROM @extschema@.part_config 
 WHERE parent_table = p_parent_table
 AND (partition_type = 'time' OR partition_type = 'time-custom');
@@ -59,13 +66,14 @@ IF NOT FOUND THEN
     RAISE EXCEPTION 'ERROR: no config found for %', p_parent_table;
 END IF;
 
+SELECT current_setting('search_path') INTO v_old_search_path;
 IF v_jobmon THEN
-    SELECT nspname INTO v_jobmon_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
+    SELECT nspname INTO v_jobmon_schema FROM pg_catalog.pg_namespace n, pg_catalog.pg_extension e WHERE e.extname = 'pg_jobmon'::name AND e.extnamespace = n.oid;
     IF v_jobmon_schema IS NOT NULL THEN
-        SELECT current_setting('search_path') INTO v_old_search_path;
-        EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', '@extschema@,'||v_jobmon_schema, 'false');
+        v_new_search_path := '@extschema@,'||v_jobmon_schema||',pg_temp';
     END IF;
 END IF;
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_new_search_path, 'false');
 
 IF v_jobmon_schema IS NOT NULL THEN
     IF p_job_id IS NULL THEN
@@ -76,7 +84,10 @@ IF v_jobmon_schema IS NOT NULL THEN
     v_step_id := add_step(v_job_id, format('Creating partition function for table %s', p_parent_table));
 END IF;
 
-SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename FROM pg_tables WHERE schemaname ||'.'|| tablename = p_parent_table;
+SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename
+FROM pg_catalog.pg_tables
+WHERE schemaname = split_part(p_parent_table, '.', 1)::name
+AND tablename = split_part(p_parent_table, '.', 2)::name;
 
 v_function_name := @extschema@.check_name_length(v_parent_tablename, '_part_trig_func', FALSE);
 
@@ -174,10 +185,10 @@ IF v_type = 'time' THEN
                 , v_control
                 , v_next_partition_timestamp);
     END IF;
-        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema AND tablename = v_current_partition_name;
+        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema::name AND tablename = v_current_partition_name::name;
         IF v_count > 0 THEN
             v_trig_func := v_trig_func || format('
-                INSERT INTO %I.%I VALUES (NEW.*); ', v_parent_schema, v_current_partition_name);
+                INSERT INTO %I.%I VALUES (NEW.*) %s; ', v_parent_schema, v_current_partition_name, v_upsert);
         ELSE
             v_trig_func := v_trig_func || '
                 -- Child table for current values does not exist in this partition set, so write to parent
@@ -192,54 +203,56 @@ IF v_type = 'time' THEN
 
         -- Check that child table exist before making a rule to insert to them.
         -- Handles optimize_trigger being larger than premake (to go back in time further) and edge case of changing optimize_trigger immediately after running create_parent().
-        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema AND tablename = v_prev_partition_name;
+        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema::name AND tablename = v_prev_partition_name::name;
         IF v_count > 0 THEN
             IF v_epoch = false THEN
                 v_trig_func := v_trig_func ||format('
             ELSIF NEW.%I >= %L AND NEW.%I < %L THEN 
-                INSERT INTO %I.%I VALUES (NEW.*);'
+                INSERT INTO %I.%I VALUES (NEW.*) %s;'
                     , v_control
                     , v_prev_partition_timestamp
                     , v_control
                     , v_prev_partition_timestamp + v_partition_interval::interval
                     , v_parent_schema
-                    , v_prev_partition_name);
+                    , v_prev_partition_name
+                    , v_upsert);
             ELSE
                 v_trig_func := v_trig_func ||format('
             ELSIF to_timestamp(NEW.%I) >= %L AND to_timestamp(NEW.%I) < %L THEN 
-                INSERT INTO %I.%I VALUES (NEW.*);'
+                INSERT INTO %I.%I VALUES (NEW.*) %s;'
                     , v_control
                     , v_prev_partition_timestamp
                     , v_control
                     , v_prev_partition_timestamp + v_partition_interval::interval
                     , v_parent_schema
-                    , v_prev_partition_name);
-
+                    , v_prev_partition_name
+                    , v_upsert);
             END IF;
         END IF;
-        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema AND tablename = v_next_partition_name;
+        SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = v_parent_schema::name AND tablename = v_next_partition_name::name;
         IF v_count > 0 THEN
             IF v_epoch = false THEN
                 v_trig_func := v_trig_func ||format(' 
             ELSIF NEW.%I >= %L AND NEW.%I < %L THEN 
-                INSERT INTO %I.%I VALUES (NEW.*);'
+                INSERT INTO %I.%I VALUES (NEW.*) %s;'
                     , v_control
                     , v_next_partition_timestamp
                     , v_control
                     , v_final_partition_timestamp
                     , v_parent_schema
-                    , v_next_partition_name);
+                    , v_next_partition_name
+                    , v_upsert);
             ELSE
                 v_trig_func := v_trig_func ||format(' 
             ELSIF to_timestamp(NEW.%I) >= %L AND to_timestamp(NEW.%I) < %L THEN 
-                INSERT INTO %I.%I VALUES (NEW.*);'
+                INSERT INTO %I.%I VALUES (NEW.*) %s;'
                     , v_control
                     , v_next_partition_timestamp
                     , v_control
                     , v_final_partition_timestamp
                     , v_parent_schema
-                    , v_next_partition_name);
-
+                    , v_next_partition_name
+                    , v_upsert);
             END IF;
         END IF;
 
@@ -248,9 +261,9 @@ IF v_type = 'time' THEN
     v_trig_func := v_trig_func||format('
             ELSE
                 v_partition_name := @extschema@.check_name_length(%L, to_char(v_partition_timestamp, %L), TRUE);
-                SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = %L AND tablename = v_partition_name;
+                SELECT count(*) INTO v_count FROM pg_catalog.pg_tables WHERE schemaname = %L::name AND tablename = v_partition_name::name;
                 IF v_count > 0 THEN 
-                    EXECUTE format(''INSERT INTO %%I.%%I VALUES($1.*)'', %L, v_partition_name) USING NEW;
+                    EXECUTE format(''INSERT INTO %%I.%%I VALUES($1.*) %s'', %L, v_partition_name) USING NEW;
                 ELSE
                     RETURN NEW;
                 END IF;
@@ -258,11 +271,19 @@ IF v_type = 'time' THEN
             , v_parent_tablename
             , v_datetime_string
             , v_parent_schema
+            , v_upsert
             , v_parent_schema);
 
     v_trig_func := v_trig_func ||'
         END IF; 
-        RETURN NULL; 
+        RETURN NULL;'; 
+    IF v_trigger_exception_handling THEN 
+        v_trig_func := v_trig_func ||'
+        EXCEPTION WHEN OTHERS THEN
+            RAISE WARNING ''pg_partman insert into child table failed, row inserted into parent (%.%). ERROR: %'', TG_TABLE_SCHEMA, TG_TABLE_NAME, COALESCE(SQLERRM, ''unknown'');
+            RETURN NEW;';
+    END IF;
+    v_trig_func := v_trig_func ||'
         END $t$;';
 
     EXECUTE v_trig_func;
@@ -280,6 +301,7 @@ ELSIF v_type = 'time-custom' THEN
             v_child_schemaname  text;
             v_child_table       text;
             v_child_tablename   text;
+            v_upsert            text;
         BEGIN
             '
         , v_parent_schema
@@ -288,20 +310,22 @@ ELSIF v_type = 'time-custom' THEN
     IF v_epoch = false THEN
         v_trig_func := v_trig_func || format(' 
 
-        SELECT child_table INTO v_child_table
-        FROM @extschema@.custom_time_partitions 
-        WHERE partition_range @> NEW.%I 
-        AND parent_table = %L;'
+        SELECT c.child_table, p.upsert INTO v_child_table, v_upsert
+        FROM @extschema@.custom_time_partitions c
+        JOIN @extschema@.part_config p ON c.parent_table = p.parent_table
+        WHERE c.partition_range @> NEW.%I 
+        AND c.parent_table = %L;'
         , v_control
         , v_parent_schema||'.'||v_parent_tablename);
 
     ELSE -- epoch true
         v_trig_func := v_trig_func || format(' 
 
-        SELECT child_table INTO v_child_table
-        FROM @extschema@.custom_time_partitions 
-        WHERE partition_range @> to_timestamp(NEW.%I)
-        AND parent_table = %L;'
+        SELECT c.child_table, p.upsert INTO v_child_table, v_upsert
+        FROM @extschema@.custom_time_partitions c
+        JOIN @extschema@.part_config p ON c.parent_table = p.parent_table
+        WHERE c.partition_range @> to_timestamp(NEW.%I)
+        AND c.parent_table = %L;'
         , v_control
         , v_parent_schema||'.'||v_parent_tablename);
 
@@ -309,14 +333,24 @@ ELSIF v_type = 'time-custom' THEN
 
     v_trig_func := v_trig_func || '
 
-        SELECT schemaname, tablename INTO v_child_schemaname, v_child_tablename FROM pg_catalog.pg_tables WHERE schemaname ||''.''|| tablename = v_child_table;
+        SELECT schemaname, tablename INTO v_child_schemaname, v_child_tablename 
+        FROM pg_catalog.pg_tables 
+        WHERE schemaname = split_part(v_child_table, ''.'', 1)::name
+        AND tablename = split_part(v_child_table, ''.'', 2)::name;
         IF v_child_schemaname IS NOT NULL AND v_child_tablename IS NOT NULL THEN
-            EXECUTE format(''INSERT INTO %I.%I VALUES ($1.*)'', v_child_schemaname, v_child_tablename) USING NEW;
+            EXECUTE format(''INSERT INTO %I.%I VALUES ($1.*) %s'', v_child_schemaname, v_child_tablename, v_upsert) USING NEW;
         ELSE
             RETURN NEW;
         END IF;
 
-        RETURN NULL; 
+        RETURN NULL;';
+    IF v_trigger_exception_handling THEN 
+        v_trig_func := v_trig_func ||'
+        EXCEPTION WHEN OTHERS THEN
+            RAISE WARNING ''pg_partman insert into child table failed, row inserted into parent (%.%). ERROR: %'', TG_TABLE_SCHEMA, TG_TABLE_NAME, COALESCE(SQLERRM, ''unknown'');
+            RETURN NEW;';
+    END IF;
+    v_trig_func := v_trig_func ||'
         END $t$;';
 
     EXECUTE v_trig_func;
@@ -331,9 +365,10 @@ END IF;
 
 IF v_jobmon_schema IS NOT NULL THEN
     PERFORM close_job(v_job_id);
-    EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
 END IF;
 
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
+
 EXCEPTION
     WHEN OTHERS THEN
         GET STACKED DIAGNOSTICS ex_message = MESSAGE_TEXT,
@@ -357,4 +392,3 @@ HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
 
-
