@@ -12,6 +12,7 @@ CREATE FUNCTION create_parent(
     , p_start_partition text DEFAULT NULL
     , p_inherit_fk boolean DEFAULT true
     , p_epoch boolean DEFAULT false
+    , p_upsert text DEFAULT ''
     , p_jobmon boolean DEFAULT true
     , p_debug boolean DEFAULT false) 
 RETURNS boolean 
@@ -23,40 +24,39 @@ ex_context                      text;
 ex_detail                       text;
 ex_hint                         text;
 ex_message                      text;
-v_base_timestamp                timestamp;
+v_base_timestamp                timestamptz;
 v_count                         int := 1;
 v_datetime_string               text;
-v_higher_parent                 text := p_parent_table;
+v_higher_parent_schema          text := split_part(p_parent_table, '.', 1);
+v_higher_parent_table           text := split_part(p_parent_table, '.', 2);
 v_id_interval                   bigint;
-v_id_position                   int;
 v_job_id                        bigint;
 v_jobmon_schema                 text;
 v_last_partition_created        boolean;
 v_max                           bigint;
 v_notnull                       boolean;
+v_new_search_path               text := '@extschema@,pg_temp';
 v_old_search_path               text;
 v_parent_partition_id           bigint;
-v_parent_partition_timestamp    timestamp;
+v_parent_partition_timestamp    timestamptz;
 v_parent_schema                 text;
 v_parent_tablename              text;
-v_partition_time                timestamp;
-v_partition_time_array          timestamp[];
+v_partition_time                timestamptz;
+v_partition_time_array          timestamptz[];
 v_partition_id_array            bigint[];
 v_row                           record;
 v_run_maint                     boolean;
 v_sql                           text;
-v_start_time                    timestamp;
+v_start_time                    timestamptz;
 v_starting_partition_id         bigint;
 v_step_id                       bigint;
 v_step_overflow_id              bigint;
 v_sub_parent                    text;
 v_success                       boolean := false;
 v_time_interval                 interval;
-v_time_position                 int;
 v_top_datetime_string           text;
-v_top_parent                    text := p_parent_table;
-v_top_schemaname                text;
-v_top_tablename                 text;
+v_top_parent_schema             text := split_part(p_parent_table, '.', 1);
+v_top_parent_table              text := split_part(p_parent_table, '.', 2);
 
 BEGIN
 
@@ -64,18 +64,25 @@ IF position('.' in p_parent_table) = 0  THEN
     RAISE EXCEPTION 'Parent table must be schema qualified';
 END IF;
 
-SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename FROM pg_catalog.pg_tables WHERE schemaname || '.' || tablename = p_parent_table;
+IF p_upsert <> '' AND @extschema@.check_version('9.5.0') = 'false' THEN
+    RAISE EXCEPTION 'INSERT ... ON CONFLICT (UPSERT) feature is only supported in PostgreSQL 9.5 and later';
+END IF;
+
+SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename
+FROM pg_catalog.pg_tables
+WHERE schemaname = split_part(p_parent_table, '.', 1)::name
+AND tablename = split_part(p_parent_table, '.', 2)::name;
     IF v_parent_tablename IS NULL THEN
-        RAISE EXCEPTION 'Please create given parent table first: %', p_parent_table;
+        RAISE EXCEPTION 'Unable to find given parent table in system catalogs. Please create parent table first: %', p_parent_table;
     END IF;
 
 SELECT attnotnull INTO v_notnull 
 FROM pg_catalog.pg_attribute a
 JOIN pg_catalog.pg_class c ON a.attrelid = c.oid
 JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-WHERE c.relname = v_parent_tablename
-AND n.nspname = v_parent_schema
-AND a.attname = p_control;
+WHERE c.relname = v_parent_tablename::name
+AND n.nspname = v_parent_schema::name
+AND a.attname = p_control::name;
     IF v_notnull = false OR v_notnull IS NULL THEN
         RAISE EXCEPTION 'Control column given (%) for parent table (%) does not exist or must be set to NOT NULL', p_control, p_parent_table;
     END IF;
@@ -88,15 +95,14 @@ IF NOT @extschema@.check_partition_type(p_type) THEN
     RAISE EXCEPTION '% is not a valid partitioning type', p_type;
 END IF;
 
-EXECUTE format('LOCK TABLE %I.%I IN ACCESS EXCLUSIVE MODE', v_parent_schema, v_parent_tablename);
-
+SELECT current_setting('search_path') INTO v_old_search_path;
 IF p_jobmon THEN
-    SELECT nspname INTO v_jobmon_schema FROM pg_namespace n, pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
+    SELECT nspname INTO v_jobmon_schema FROM pg_catalog.pg_namespace n, pg_catalog.pg_extension e WHERE e.extname = 'pg_jobmon'::name AND e.extnamespace = n.oid;
     IF v_jobmon_schema IS NOT NULL THEN
-        SELECT current_setting('search_path') INTO v_old_search_path;
-        EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', '@extschema@,'||v_jobmon_schema, 'false');
+        v_new_search_path := '@extschema@,'||v_jobmon_schema||',pg_temp';
     END IF;
 END IF;
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_new_search_path, 'false');
 
 IF p_use_run_maintenance IS NOT NULL THEN
     IF p_use_run_maintenance IS FALSE AND (p_type = 'time' OR p_type = 'time-custom') THEN
@@ -111,6 +117,8 @@ ELSE
     RAISE EXCEPTION 'use_run_maintenance value cannot be set NULL';
 END IF;
 
+EXECUTE format('LOCK TABLE %I.%I IN ACCESS EXCLUSIVE MODE', v_parent_schema, v_parent_tablename);
+
 IF v_jobmon_schema IS NOT NULL THEN
     v_job_id := add_job(format('PARTMAN SETUP PARENT: %s', p_parent_table));
     v_step_id := add_step(v_job_id, format('Creating initial partitions on new parent table: %s', p_parent_table));
@@ -124,8 +132,8 @@ FOR v_row IN
         FROM pg_catalog.pg_inherits h
         JOIN pg_catalog.pg_class c ON h.inhrelid = c.oid
         JOIN pg_catalog.pg_namespace n ON c.relnamespace = n.oid
-        WHERE c.relname = v_parent_tablename
-        AND n.nspname = v_parent_schema
+        WHERE c.relname = v_parent_tablename::name
+        AND n.nspname = v_parent_schema::name
     ), sibling_children AS (
         SELECT i.inhrelid::regclass::text AS tablename 
         FROM pg_inherits i
@@ -145,7 +153,10 @@ FOR v_row IN
         , sub_epoch
         , sub_optimize_trigger
         , sub_optimize_constraint
+        , sub_infinite_time_partitions
         , sub_jobmon
+        , sub_trigger_exception_handling
+        , sub_upsert
     FROM @extschema@.part_config_sub a
     JOIN sibling_children b on a.sub_parent = b.tablename LIMIT 1
 LOOP
@@ -165,7 +176,10 @@ LOOP
         , sub_epoch
         , sub_optimize_trigger
         , sub_optimize_constraint
-        , sub_jobmon)
+        , sub_infinite_time_partitions
+        , sub_jobmon
+        , sub_trigger_exception_handling
+        , sub_upsert)
     VALUES (
         p_parent_table
         , v_row.sub_partition_type
@@ -182,7 +196,10 @@ LOOP
         , v_row.sub_epoch
         , v_row.sub_optimize_trigger
         , v_row.sub_optimize_constraint
-        , v_row.sub_jobmon);
+        , v_row.sub_infinite_time_partitions
+        , v_row.sub_jobmon
+        , v_row.sub_trigger_exception_handling
+        , v_row.sub_upsert);
 END LOOP;
 
 IF p_type = 'time' OR p_type = 'time-custom' THEN
@@ -215,7 +232,7 @@ IF p_type = 'time' OR p_type = 'time-custom' THEN
     END CASE;
 
     -- First partition is either the min premake or p_start_partition
-    v_start_time := COALESCE(p_start_partition::timestamp, CURRENT_TIMESTAMP - (v_time_interval * p_premake));
+    v_start_time := COALESCE(p_start_partition::timestamptz, CURRENT_TIMESTAMP - (v_time_interval * p_premake));
 
     IF v_time_interval >= '1 year' THEN
         v_base_timestamp := date_trunc('year', v_start_time);
@@ -263,7 +280,7 @@ IF p_type = 'time' OR p_type = 'time-custom' THEN
         -- If current loop value is less than or equal to the value of the max premake, add time to array.
         IF (v_base_timestamp + (v_time_interval * v_count)) < (CURRENT_TIMESTAMP + (v_time_interval * p_premake)) THEN
             BEGIN
-                v_partition_time := (v_base_timestamp + (v_time_interval * v_count))::timestamp;
+                v_partition_time := (v_base_timestamp + (v_time_interval * v_count))::timestamptz;
                 v_partition_time_array := array_append(v_partition_time_array, v_partition_time);
             EXCEPTION WHEN datetime_field_overflow THEN
                 RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range. 
@@ -289,7 +306,8 @@ IF p_type = 'time' OR p_type = 'time-custom' THEN
         , datetime_string
         , use_run_maintenance
         , inherit_fk
-        , jobmon) 
+        , jobmon 
+        , upsert)
     VALUES (
         p_parent_table
         , p_type
@@ -301,7 +319,9 @@ IF p_type = 'time' OR p_type = 'time-custom' THEN
         , v_datetime_string
         , v_run_maint
         , p_inherit_fk
-        , p_jobmon);
+        , p_jobmon
+        , p_upsert);
+
     v_last_partition_created := @extschema@.create_partition_time(p_parent_table, v_partition_time_array, false);
 
     IF v_last_partition_created = false THEN 
@@ -312,18 +332,17 @@ IF p_type = 'time' OR p_type = 'time-custom' THEN
             FROM pg_catalog.pg_inherits i
             JOIN pg_catalog.pg_class c ON c.oid = i.inhrelid
             JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
-            WHERE c.relname = v_parent_tablename
-            AND n.nspname = v_parent_schema
-        ) SELECT n.nspname||'.'||c.relname, p.datetime_string
-        INTO v_top_parent, v_top_datetime_string
+            WHERE c.relname = v_parent_tablename::name
+            AND n.nspname = v_parent_schema::name
+        ) SELECT n.nspname, c.relname 
+        INTO v_top_parent_schema, v_top_parent_table 
         FROM pg_catalog.pg_class c
         JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
         JOIN top_oid t ON c.oid = t.top_parent_oid
         JOIN @extschema@.part_config p ON p.parent_table = n.nspname||'.'||c.relname;
-        IF v_top_parent IS NOT NULL THEN
+        IF v_top_parent_table IS NOT NULL THEN
             -- If so create the lowest possible partition that is within the boundary of the parent
-            v_time_position := (length(p_parent_table) - position('p_' in reverse(p_parent_table))) + 2;
-            v_parent_partition_timestamp := to_timestamp(substring(p_parent_table from v_time_position), v_top_datetime_string);
+            SELECT child_start_time INTO v_parent_partition_timestamp FROM @extschema@.show_partition_info(p_parent_table, p_parent_table := v_top_parent_schema||'.'||v_top_parent_table);
             IF v_base_timestamp >= v_parent_partition_timestamp THEN
                 WHILE v_base_timestamp >= v_parent_partition_timestamp LOOP
                     v_base_timestamp := v_base_timestamp - v_time_interval;
@@ -356,36 +375,40 @@ IF p_type = 'id' THEN
     END IF;
 
     -- Check if parent table is a subpartition of an already existing id partition set managed by pg_partman. 
-    WHILE v_higher_parent IS NOT NULL LOOP -- initially set in DECLARE
+    WHILE v_higher_parent_table IS NOT NULL LOOP -- initially set in DECLARE
         WITH top_oid AS (
             SELECT i.inhparent AS top_parent_oid
             FROM pg_catalog.pg_inherits i
             JOIN pg_catalog.pg_class c ON c.oid = i.inhrelid
             JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
-            WHERE n.nspname||'.'||c.relname = v_higher_parent
-        ) SELECT n.nspname||'.'||c.relname
-        INTO v_higher_parent
+            WHERE n.nspname = v_higher_parent_schema::name
+            AND c.relname = v_higher_parent_table::name
+        ) SELECT n.nspname, c.relname
+        INTO v_higher_parent_schema, v_higher_parent_table
         FROM pg_catalog.pg_class c
         JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
         JOIN top_oid t ON c.oid = t.top_parent_oid
         JOIN @extschema@.part_config p ON p.parent_table = n.nspname||'.'||c.relname
         WHERE p.partition_type = 'id';
 
-        IF v_higher_parent IS NOT NULL THEN
+        IF v_higher_parent_table IS NOT NULL THEN
             -- v_top_parent initially set in DECLARE
-            v_top_parent := v_higher_parent;
+            v_top_parent_schema := v_higher_parent_schema;
+            v_top_parent_table := v_higher_parent_table;
         END IF;
     END LOOP;
 
     -- If custom start partition is set, use that.
     -- If custom start is not set and there is already data, start partitioning with the highest current value and ensure it's grabbed from highest top parent table
-    SELECT schemaname, tablename INTO v_top_schemaname, v_top_tablename FROM pg_catalog.pg_tables WHERE schemaname||'.'||tablename = v_top_parent;
-    v_sql := format('SELECT COALESCE(%L, max(%I)::bigint, 0) FROM %I.%I LIMIT 1'
-                , p_start_partition::bigint
-                , p_control
-                , v_top_schemaname
-                , v_top_tablename);
-    EXECUTE v_sql INTO v_max;
+    IF p_start_partition IS NOT NULL THEN
+        v_max := p_start_partition::bigint;
+    ELSE
+        v_sql := format('SELECT COALESCE(max(%I)::bigint, 0) FROM %I.%I LIMIT 1'
+                    , p_control
+                    , v_top_parent_schema
+                    , v_top_parent_table);
+        EXECUTE v_sql INTO v_max;
+    END IF;
     v_starting_partition_id := v_max - (v_max % v_id_interval);
     FOR i IN 0..p_premake LOOP
         -- Only make previous partitions if ID value is less than the starting value and positive (and custom start partition wasn't set)
@@ -407,7 +430,8 @@ IF p_type = 'id' THEN
         , constraint_cols
         , use_run_maintenance
         , inherit_fk
-        , jobmon) 
+        , jobmon
+        , upsert) 
     VALUES (
         p_parent_table
         , p_type
@@ -417,7 +441,8 @@ IF p_type = 'id' THEN
         , p_constraint_cols
         , v_run_maint
         , p_inherit_fk
-        , p_jobmon);
+        , p_jobmon
+        , p_upsert);
     v_last_partition_created := @extschema@.create_partition_id(p_parent_table, v_partition_id_array, false);
     IF v_last_partition_created = false THEN
         -- This can happen with subpartitioning when future or past partitions prevent child creation because they're out of range of the parent
@@ -427,19 +452,18 @@ IF p_type = 'id' THEN
             FROM pg_catalog.pg_inherits i
             JOIN pg_catalog.pg_class c ON c.oid = i.inhrelid
             JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
-            WHERE c.relname = v_parent_tablename
-            AND n.nspname = v_parent_schema
+            WHERE c.relname = v_parent_tablename::name
+            AND n.nspname = v_parent_schema::name
         ) SELECT n.nspname||'.'||c.relname
-        INTO v_top_parent
+        INTO v_top_parent_table
         FROM pg_catalog.pg_class c
         JOIN pg_catalog.pg_namespace n ON n.oid = c.relnamespace
         JOIN top_oid t ON c.oid = t.top_parent_oid
         JOIN @extschema@.part_config p ON p.parent_table = n.nspname||'.'||c.relname
         WHERE p.partition_type = 'id';
-        IF v_top_parent IS NOT NULL THEN
+        IF v_top_parent_table IS NOT NULL THEN
             -- Create the lowest possible partition that is within the boundary of the parent
-            v_id_position := (length(p_parent_table) - position('p_' in reverse(p_parent_table))) + 2;
-            v_parent_partition_id = substring(p_parent_table from v_id_position)::bigint;
+             SELECT child_start_id INTO v_parent_partition_id FROM @extschema@.show_partition_info(p_parent_table, p_parent_table := v_top_parent_table);
             IF v_starting_partition_id >= v_parent_partition_id THEN
                 WHILE v_starting_partition_id >= v_parent_partition_id LOOP
                     v_starting_partition_id := v_starting_partition_id - v_id_interval;
@@ -488,9 +512,10 @@ IF v_jobmon_schema IS NOT NULL THEN
     ELSE
         PERFORM close_job(v_job_id);
     END IF;
-    EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
 END IF;
 
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
+
 v_success := true;
 
 RETURN v_success;
@@ -518,3 +543,4 @@ HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
 
+
