@@ -13,17 +13,19 @@ v_epoch                     boolean;
 v_last_partition            text;
 v_lock_iter                 int := 1;
 v_lock_obtained             boolean := FALSE;
-v_max_partition_timestamp   timestamp;
-v_min_partition_timestamp   timestamp;
+v_max_partition_timestamp   timestamptz;
+v_min_partition_timestamp   timestamptz;
+v_new_search_path           text := '@extschema@,pg_temp';
+v_old_search_path           text;
 v_parent_schema             text;
 v_parent_tablename          text;
 v_partition_interval        interval;
 v_partition_suffix          text;
-v_partition_timestamp       timestamp[];
+v_partition_timestamp       timestamptz[];
 v_quarter                   text;
 v_rowcount                  bigint;
 v_sql                       text;
-v_start_control             timestamp;
+v_start_control             timestamptz;
 v_time_position             int;
 v_total_rows                bigint := 0;
 v_type                      text;
@@ -31,6 +33,9 @@ v_year                      text;
 
 BEGIN
 
+SELECT current_setting('search_path') INTO v_old_search_path;
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_new_search_path, 'false');
+
 SELECT partition_type
     , partition_interval::interval
     , control
@@ -45,6 +50,7 @@ FROM @extschema@.part_config
 WHERE parent_table = p_parent_table
 AND (partition_type = 'time' OR partition_type = 'time-custom');
 IF NOT FOUND THEN
+    EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
     RAISE EXCEPTION 'ERROR: no config found for %', p_parent_table;
 END IF;
 
@@ -52,8 +58,12 @@ IF p_batch_interval IS NULL OR p_batch_interval > v_partition_interval THEN
     p_batch_interval := v_partition_interval;
 END IF;
 
+SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename
+FROM pg_catalog.pg_tables
+WHERE schemaname = split_part(p_parent_table, '.', 1)::name
+AND tablename = split_part(p_parent_table, '.', 2)::name;
+
 SELECT partition_tablename INTO v_last_partition FROM @extschema@.show_partitions(p_parent_table, 'DESC') LIMIT 1;
-SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename FROM pg_catalog.pg_tables WHERE schemaname ||'.'|| tablename = p_parent_table;
 
 FOR i IN 1..p_batch_count LOOP
 
@@ -101,17 +111,25 @@ FOR i IN 1..p_batch_count LOOP
                 v_min_partition_timestamp := date_trunc('year', v_start_control);
         END CASE;
     ELSIF v_type = 'time-custom' THEN
-        -- Keep going backwards, checking if the time interval encompases the current v_start_control value
-        v_time_position := (length(v_last_partition) - position('p_' in reverse(v_last_partition))) + 2;
-        v_min_partition_timestamp := to_timestamp(substring(v_last_partition from v_time_position), v_datetime_string);
+        SELECT child_start_time INTO v_min_partition_timestamp FROM @extschema@.show_partition_info(v_parent_schema||'.'||v_last_partition
+            , v_partition_interval::text
+            , p_parent_table);
         v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
         LOOP
             IF v_start_control >= v_min_partition_timestamp AND v_start_control < v_max_partition_timestamp THEN
                 EXIT;
             ELSE
-                v_max_partition_timestamp := v_min_partition_timestamp;
                 BEGIN
-                    v_min_partition_timestamp := v_min_partition_timestamp - v_partition_interval;
+                    IF v_start_control > v_max_partition_timestamp THEN
+                        -- Keep going forward in time, checking if child partition time interval encompasses the current v_start_control value
+                        v_min_partition_timestamp := v_max_partition_timestamp;
+                        v_max_partition_timestamp := v_max_partition_timestamp + v_partition_interval;
+
+                    ELSE
+                        -- Keep going backwards in time, checking if child partition time interval encompasses the current v_start_control value
+                        v_max_partition_timestamp := v_min_partition_timestamp;
+                        v_min_partition_timestamp := v_min_partition_timestamp - v_partition_interval;
+                    END IF;
                 EXCEPTION WHEN datetime_field_overflow THEN
                     RAISE EXCEPTION 'Attempted partition time interval is outside PostgreSQL''s supported time range. 
                         Unable to create partition with interval before timestamp % ', v_min_partition_interval;
@@ -123,6 +141,7 @@ FOR i IN 1..p_batch_count LOOP
 
     v_partition_timestamp := ARRAY[v_min_partition_timestamp];
     IF p_order = 'ASC' THEN
+        -- Ensure batch interval given as parameter doesn't cause maximum to overflow the current partition maximum
         IF (v_start_control + p_batch_interval) >= (v_min_partition_timestamp + v_partition_interval) THEN
             v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
         ELSE
@@ -131,7 +150,7 @@ FOR i IN 1..p_batch_count LOOP
     ELSIF p_order = 'DESC' THEN
         -- Must be greater than max value still in parent table since query below grabs < max
         v_max_partition_timestamp := v_min_partition_timestamp + v_partition_interval;
-        -- Make sure minimum doesn't underflow current partition minimum
+        -- Ensure batch interval given as parameter doesn't cause minimum to underflow current partition minimum
         IF (v_start_control - p_batch_interval) >= v_min_partition_timestamp THEN
             v_min_partition_timestamp = v_start_control - p_batch_interval;
         END IF;
@@ -217,8 +236,11 @@ END LOOP;
 
 PERFORM @extschema@.create_function_time(p_parent_table);
 
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
+
 RETURN v_total_rows;
 
 END
 $$;
 
+
