@@ -20,29 +20,29 @@ v_check_subpart                 int;
 v_create_count                  int := 0;
 v_current_partition             text;
 v_current_partition_id          bigint;
-v_current_partition_timestamp   timestamp;
+v_current_partition_timestamp   timestamptz;
 v_datetime_string               text;
 v_drop_count                    int := 0;
-v_id_position                   int;
 v_job_id                        bigint;
 v_jobmon                        boolean;
 v_jobmon_schema                 text;
 v_last_partition                text;
 v_last_partition_created        boolean;
 v_last_partition_id             bigint;
-v_last_partition_timestamp      timestamp;
+v_last_partition_timestamp      timestamptz;
 v_max_id_parent                 bigint;
-v_max_time_parent               timestamp;
+v_max_time_parent               timestamptz;
+v_new_search_path               text := '@extschema@,pg_temp';
 v_next_partition_id             bigint;
-v_next_partition_timestamp      timestamp;
+v_next_partition_timestamp      timestamptz;
+v_old_search_path               text;
 v_parent_schema                 text;
 v_parent_tablename              text;
 v_premade_count                 int;
 v_premake_id_max                bigint;
 v_premake_id_min                bigint;
-v_premake_timestamp_min         timestamp;
-v_premake_timestamp_max         timestamp;
-v_quarter                       text;
+v_premake_timestamp_min         timestamptz;
+v_premake_timestamp_max         timestamptz;
 v_row                           record;
 v_row_max_id                    record;
 v_row_max_time                  record;
@@ -50,18 +50,15 @@ v_row_sub                       record;
 v_skip_maint                    boolean;
 v_step_id                       bigint;
 v_step_overflow_id              bigint;
-v_step_serial_id                bigint;
 v_sub_id_max                    bigint;
 v_sub_id_max_suffix             bigint;
 v_sub_id_min                    bigint;
 v_sub_parent                    text;
-v_sub_timestamp_max             timestamp;
-v_sub_timestamp_max_suffix      timestamp;
-v_sub_timestamp_min             timestamp;
+v_sub_timestamp_max             timestamptz;
+v_sub_timestamp_max_suffix      timestamptz;
+v_sub_timestamp_min             timestamptz;
 v_tablename                     text;
 v_tables_list_sql               text;
-v_time_position                 int;
-v_year                          text;
 
 BEGIN
 
@@ -71,33 +68,22 @@ IF v_adv_lock = 'false' THEN
     RETURN;
 END IF;
 
+SELECT current_setting('search_path') INTO v_old_search_path;
 IF p_jobmon THEN
-    SELECT nspname INTO v_jobmon_schema FROM pg_catalog.pg_namespace n, pg_catalog.pg_extension e WHERE e.extname = 'pg_jobmon' AND e.extnamespace = n.oid;
+    SELECT nspname INTO v_jobmon_schema FROM pg_catalog.pg_namespace n, pg_catalog.pg_extension e WHERE e.extname = 'pg_jobmon'::name AND e.extnamespace = n.oid;
+    IF v_jobmon_schema IS NOT NULL THEN
+        v_new_search_path := '@extschema@,'||v_jobmon_schema||',pg_temp';
+    END IF;
 END IF;
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_new_search_path, 'false');
 
 IF v_jobmon_schema IS NOT NULL THEN
-    EXECUTE format('SELECT %I.add_job(%L)', v_jobmon_schema, 'PARTMAN RUN MAINTENANCE') INTO v_job_id;
-    EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'Running maintenance loop') INTO v_step_id;
+    v_job_id := add_job('PARTMAN RUN MAINTENANCE');
+    v_step_id := add_step(v_job_id, 'Running maintenance loop');
 END IF;
 
--- Check for consistent data in part_config_sub table. Was unable to get this working properly as either a constraint or trigger. 
--- Would either delay raising an error until the next write (which I cannot predict) or disallow future edits to update a sub-partition set's configuration.
--- This way at least provides a consistent way to check that I know will run. If anyone can get a working constraint/trigger, please help!
--- Don't have to worry about this in the serial trigger maintenance since subpartitioning requires run_maintenance().
-FOR v_row IN 
-    SELECT sub_parent FROM @extschema@.part_config_sub
-LOOP
-    SELECT count(*) INTO v_check_subpart FROM @extschema@.check_subpart_sameconfig(v_row.sub_parent);
-    IF v_check_subpart > 1 THEN
-        RAISE EXCEPTION 'Inconsistent data in part_config_sub table. Sub-partition tables that are themselves sub-partitions cannot have differing configuration values among their siblings. 
-        Run this query: "SELECT * FROM @extschema@.check_subpart_sameconfig(''%'');" This should only return a single row or nothing. 
-        If multiple rows are returned, results are all children of the given parent. Update the differing values to be consistent for your desired values.', v_row.sub_parent;
-    END IF;
-END LOOP;
-
 v_row := NULL; -- Ensure it's reset
 
-
 v_tables_list_sql := 'SELECT parent_table
                 , partition_type
                 , partition_interval
@@ -107,6 +93,7 @@ v_tables_list_sql := 'SELECT parent_table
                 , undo_in_progress
                 , sub_partition_set_full
                 , epoch
+                , infinite_time_partitions
             FROM @extschema@.part_config
             WHERE sub_partition_set_full = false';
 
@@ -122,7 +109,25 @@ LOOP
     CONTINUE WHEN v_row.undo_in_progress;
     v_skip_maint := true; -- reset every loop
 
-    SELECT schemaname, tablename INTO v_parent_schema, v_parent_tablename FROM pg_catalog.pg_tables WHERE schemaname ||'.'|| tablename = v_row.parent_table;
+    -- Check for consistent data in part_config_sub table. Was unable to get this working properly as either a constraint or trigger. 
+    -- Would either delay raising an error until the next write (which I cannot predict) or disallow future edits to update a sub-partition set's configuration.
+    -- This way at least provides a consistent way to check that I know will run. If anyone can get a working constraint/trigger, please help!
+    -- Don't have to worry about this in the serial trigger maintenance since subpartitioning requires run_maintenance().
+    SELECT sub_parent INTO v_sub_parent FROM @extschema@.part_config_sub WHERE sub_parent = v_row.parent_table;
+    IF v_sub_parent IS NOT NULL THEN
+        SELECT count(*) INTO v_check_subpart FROM @extschema@.check_subpart_sameconfig(v_row.parent_table);
+        IF v_check_subpart > 1 THEN
+            RAISE EXCEPTION 'Inconsistent data in part_config_sub table. Sub-partition tables that are themselves sub-partitions cannot have differing configuration values among their siblings. 
+            Run this query: "SELECT * FROM @extschema@.check_subpart_sameconfig(''%'');" This should only return a single row or nothing. 
+            If multiple rows are returned, results are all children of the given parent. Update the differing values to be consistent for your desired values.', v_row.sub_parent;
+        END IF;
+    END IF;
+
+    SELECT schemaname, tablename 
+    INTO v_parent_schema, v_parent_tablename 
+    FROM pg_catalog.pg_tables 
+    WHERE schemaname = split_part(v_row.parent_table, '.', 1)::name
+    AND tablename = split_part(v_row.parent_table, '.', 2)::name;
 
     SELECT partition_tablename INTO v_last_partition FROM @extschema@.show_partitions(v_row.parent_table, 'DESC') LIMIT 1;
     IF p_debug THEN
@@ -131,24 +136,8 @@ LOOP
 
     IF v_row.partition_type = 'time' OR v_row.partition_type = 'time-custom' THEN
 
-        v_time_position := (length(v_last_partition) - position('p_' in reverse(v_last_partition))) + 2;
-        IF v_row.partition_interval::interval <> '3 months' OR (v_row.partition_interval::interval = '3 months' AND v_row.partition_type = 'time-custom') THEN
-           v_last_partition_timestamp := to_timestamp(substring(v_last_partition from v_time_position), v_row.datetime_string);
-        ELSE
-            -- to_timestamp doesn't recognize 'Q' date string formater. Handle it
-            v_year := split_part(substring(v_last_partition FROM v_time_position), 'q', 1);
-            v_quarter := split_part(substring(v_last_partition FROM v_time_position), 'q', 2);
-            CASE
-                WHEN v_quarter = '1' THEN
-                    v_last_partition_timestamp := to_timestamp(v_year || '-01-01', 'YYYY-MM-DD');
-                WHEN v_quarter = '2' THEN
-                    v_last_partition_timestamp := to_timestamp(v_year || '-04-01', 'YYYY-MM-DD');
-                WHEN v_quarter = '3' THEN
-                    v_last_partition_timestamp := to_timestamp(v_year || '-07-01', 'YYYY-MM-DD');
-                WHEN v_quarter = '4' THEN
-                    v_last_partition_timestamp := to_timestamp(v_year || '-10-01', 'YYYY-MM-DD');
-            END CASE;
-        END IF;
+        SELECT child_start_time INTO v_last_partition_timestamp 
+            FROM @extschema@.show_partition_info(v_parent_schema||'.'||v_last_partition, v_row.partition_interval, v_row.parent_table);
 
         -- Loop through child tables starting from highest to get current max value in partition set
         -- Avoids doing a scan on entire partition set and/or getting any values accidentally in parent.
@@ -186,13 +175,18 @@ LOOP
         IF v_max_time_parent > v_current_partition_timestamp THEN
             SELECT suffix_timestamp INTO v_current_partition_timestamp FROM @extschema@.show_partition_name(v_row.parent_table, v_max_time_parent::text);
         END IF;
-        IF v_current_partition_timestamp IS NULL THEN
-            -- Partition set is completely empty. Nothing to do
-            CONTINUE;
+        IF v_current_partition_timestamp IS NULL THEN -- Partition set is completely empty
+            IF v_row.infinite_time_partitions IS TRUE THEN
+                -- Set it to now so new partitions continue to be created
+                v_current_partition_timestamp = CURRENT_TIMESTAMP;
+            ELSE
+                -- Nothing to do
+                CONTINUE;
+            END IF;
         END IF;
 
         -- If this is a subpartition, determine if the last child table has been made. If so, mark it as full so future maintenance runs can skip it
-        SELECT sub_min::timestamp, sub_max::timestamp INTO v_sub_timestamp_min, v_sub_timestamp_max FROM @extschema@.check_subpartition_limits(v_row.parent_table, 'time');
+        SELECT sub_min::timestamptz, sub_max::timestamptz INTO v_sub_timestamp_min, v_sub_timestamp_max FROM @extschema@.check_subpartition_limits(v_row.parent_table, 'time');
         IF v_sub_timestamp_max IS NOT NULL THEN
             SELECT suffix_timestamp INTO v_sub_timestamp_max_suffix FROM @extschema@.show_partition_name(v_row.parent_table, v_sub_timestamp_max::text);
             IF v_sub_timestamp_max_suffix = v_last_partition_timestamp THEN
@@ -226,8 +220,8 @@ LOOP
             EXCEPTION WHEN datetime_field_overflow THEN
                 v_premade_count := v_row.premake; -- do this so it can exit the premake check loop and continue in the outer for loop 
                 IF v_jobmon_schema IS NOT NULL THEN
-                    EXECUTE format('SELECT %I.add_step(%L, %L)', v_jobmon_schema, v_job_id, 'Attempted partition time interval is outside PostgreSQL''s supported time range.') INTO v_step_overflow_id;
-                    EXECUTE format('SELECT %I.update_step(%L, %L, %L)', v_jobmon_schema, v_step_overflow_id, 'CRITICAL', 'Child partition creation skippd for parent table '||v_partition_time);
+                    v_step_overflow_id := add_step(v_job_id, 'Attempted partition time interval is outside PostgreSQL''s supported time range.');
+                    PERFORM update_step(v_step_overflow_id, 'CRITICAL', format('Child partition creation skipped for parent table: %s', v_partition_time));
                 END IF;
                 RAISE WARNING 'Attempted partition time interval is outside PostgreSQL''s supported time range. Child partition creation skipped for parent table %', v_row.parent_table;
                 CONTINUE;
@@ -266,8 +260,8 @@ LOOP
             CONTINUE;
         END IF;
 
-        v_id_position := (length(v_last_partition) - position('p_' in reverse(v_last_partition))) + 2;
-        v_last_partition_id = substring(v_last_partition from v_id_position)::bigint;
+        SELECT child_start_id INTO v_last_partition_id
+            FROM @extschema@.show_partition_info(v_parent_schema||'.'||v_last_partition, v_row.partition_interval, v_row.parent_table);
         -- Determine if this table is a child of a subpartition parent. If so, get limits to see if run_maintenance even needs to run for it.
         SELECT sub_min::bigint, sub_max::bigint INTO v_sub_id_min, v_sub_id_max FROM @extschema@.check_subpartition_limits(v_row.parent_table, 'id');
         IF v_sub_id_max IS NOT NULL THEN
@@ -301,9 +295,6 @@ LOOP
 
     END IF; -- end main IF check for time or id
 
-    -- Manage additonal constraints if set
-    PERFORM @extschema@.apply_constraints(p_parent_table := v_row.parent_table, p_job_id := v_job_id, p_debug := p_debug);
-
 END LOOP; -- end of creation loop
 
 -- Manage dropping old partitions if retention option is set
@@ -342,19 +333,16 @@ LOOP
 END LOOP; 
 
 IF v_jobmon_schema IS NOT NULL THEN
-    EXECUTE format('SELECT %I.update_step(%L, %L, ''Partition maintenance finished. %s partitons made. %s partitions dropped.'')'
-        , v_jobmon_schema
-        , v_step_id
-        , 'OK'
-        , v_create_count
-        , v_drop_count);
-    IF v_step_overflow_id IS NOT NULL OR v_step_serial_id IS NOT NULL THEN
-        EXECUTE format('SELECT %I.fail_job(%L)', v_jobmon_schema, v_job_id);
+    PERFORM update_step(v_step_id, 'OK', format('Partition maintenance finished. %s partitons made. %s partitions dropped.', v_create_count, v_drop_count));
+    IF v_step_overflow_id IS NOT NULL THEN
+        PERFORM fail_job(v_job_id);
     ELSE
-        EXECUTE format('SELECT %I.close_job(%L)', v_jobmon_schema, v_job_id);
+        PERFORM close_job(v_job_id);
     END IF;
 END IF;
 
+EXECUTE format('SELECT set_config(%L, %L, %L)', 'search_path', v_old_search_path, 'false');
+
 EXCEPTION
     WHEN OTHERS THEN
         GET STACKED DIAGNOSTICS ex_message = MESSAGE_TEXT,
@@ -378,3 +366,4 @@ HINT: %', ex_message, ex_context, ex_detail, ex_hint;
 END
 $$;
 
+
